@@ -108,6 +108,7 @@ sub check_canrename ($$$$$$) {
 			}
 		}
 	});
+	return defined $canrename ? $canrename : 1;
 }
 
 sub rename_form ($$$) {
@@ -125,7 +126,7 @@ sub rename_form ($$$) {
 		method => 'POST',
 		javascript => 0,
 		params => $q,
-		action => $config{cgiurl},
+		action => IkiWiki::cgiurl(),
 		stylesheet => 1,
 		fields => [qw{do page new_name attachment}],
 	);
@@ -178,8 +179,15 @@ sub rename_start ($$$$) {
 	my $attachment=shift;
 	my $page=shift;
 
-	check_canrename($page, $pagesources{$page}, undef, undef,
-		$q, $session);
+	# Special case for renaming held attachments; normal checks
+	# don't apply.
+	my $held=$attachment &&
+		IkiWiki::Plugin::attachment->can("is_held_attachment") &&
+		IkiWiki::Plugin::attachment::is_held_attachment($page);
+	if (! $held) {
+		check_canrename($page, $pagesources{$page}, undef, undef,
+			$q, $session);
+	}
 
    	# Save current form state to allow returning to it later
 	# without losing any edits.
@@ -290,13 +298,11 @@ sub sessioncgi ($$) {
 		elsif ($form->submitted eq 'Rename' && $form->validate) {
 			IkiWiki::checksessionexpiry($q, $session, $q->param('sid'));
 
-			# Queue of rename actions to perfom.
-			my @torename;
-
 			# These untaints are safe because of the checks
 			# performed in check_canrename later.
 			my $src=$form->field("page");
-			my $srcfile=IkiWiki::possibly_foolish_untaint($pagesources{$src});
+			my $srcfile=IkiWiki::possibly_foolish_untaint($pagesources{$src})
+				if exists $pagesources{$src};
 			my $dest=IkiWiki::possibly_foolish_untaint(titlepage($form->field("new_name")));
 			my $destfile=$dest;
 			if (! $q->param("attachment")) {
@@ -311,6 +317,19 @@ sub sessioncgi ($$) {
 				
 				$destfile=newpagefile($dest, $type);
 			}
+		
+			# Special case for renaming held attachments.
+			my $held=$q->param("attachment") &&
+				IkiWiki::Plugin::attachment->can("is_held_attachment") &&
+				IkiWiki::Plugin::attachment::is_held_attachment($src);
+			if ($held) {
+				rename($held, IkiWiki::Plugin::attachment::attachment_holding_location($dest));
+				postrename($session, $src, $dest, $q->param("attachment"))
+					unless defined $srcfile;
+			}
+			
+			# Queue of rename actions to perfom.
+			my @torename;
 			push @torename, {
 				src => $src,
 			       	srcfile => $srcfile,
@@ -349,8 +368,9 @@ sub sessioncgi ($$) {
 				$pagesources{$rename->{src}}=$rename->{destfile};
 			}
 			IkiWiki::rcs_commit_staged(
-				sprintf(gettext("rename %s to %s"), $srcfile, $destfile),
-				$session->param("name"), $ENV{REMOTE_ADDR}) if $config{rcs};
+				message => sprintf(gettext("rename %s to %s"), $srcfile, $destfile),
+				session => $session,
+			) if $config{rcs};
 
 			# Then link fixups.
 			foreach my $rename (@torename) {
@@ -565,6 +585,7 @@ sub fixlinks ($$$) {
 		}
 		if ($needfix) {
 			my $file=$pagesources{$page};
+			next unless -e $config{srcdir}."/".$file;
 			my $oldcontent=readfile($config{srcdir}."/".$file);
 			my $content=renamepage_hook($page, $rename->{src}, $rename->{dest}, $oldcontent);
 			if ($oldcontent ne $content) {
@@ -572,11 +593,10 @@ sub fixlinks ($$$) {
 				eval { writefile($file, $config{srcdir}, $content) };
 				next if $@;
 				my $conflict=IkiWiki::rcs_commit(
-					$file,
-					sprintf(gettext("update for rename of %s to %s"), $rename->{srcfile}, $rename->{destfile}),
-					$token,
-					$session->param("name"), 
-					$ENV{REMOTE_ADDR}
+					file => $file,
+					message => sprintf(gettext("update for rename of %s to %s"), $rename->{srcfile}, $rename->{destfile}),
+					token => $token,
+					session => $session,
 				);
 				push @fixedlinks, $page if ! defined $conflict;
 			}
